% Fig14_12.m - Simulations of diagram in Fig. 14.11

% Copyright (c) Ken Dutton, 1994

disp(' ')
disp(' If the program pauses, try looking for a plot in a')
disp(' hidden window')
disp(' ')

%
% Forward path state-space model:
% (double integrator with x1 at overall o/p, x2 at first integrator
%  o/p. The c and d matrices are set so that states x1 and x2 are
%  also defined as outputs y1 and y2. The MATLAB step command
%  then actually displays the states.)
a=[0  1;0  0]; b=[0  1]'; c=[1   0;0  1]; d=[0  0]';
%
% we need a discrete-time version for the nonlinear simulation:
ts=0.02;                  % sample period 20ms
[phi,del]=c2d(a,b,ts);    % (c and d matrices are unchanged)
%
% initialisations:
tend=9.9;      % end time for simulation
uop=1;         % operating point (used as unit step input here)
nldamp=1.5;    % nonlinear damping coefficient
yinit=[0  0]'; % initial conditions on the states and outputs (see below)
y=yinit;       % begins a matrix y which we shall use to hold the outputs
               % at every time step for plotting (note that it would lead
               % to quicker execution if y was initialised to a matrix
               % of zeros, equal to its final size. This is done in some
               % of the larger simulations on the disk - e.g. fig14_21.m)
xold=y;   % initial state values to start the simulation
t=0;      % initial time value
tnew=0;   % tnew will be used to calculate the time at each step
%
% simulation FOR FIGURE 14.12(a) (unit step simulation from zero initial
%                             conditions, to find steady operating point) 
disp('')
disp('Calculating Fig. 14.12(a) - please wait for (b) to (d)')
while tnew < tend
   tnew=t(length(t))+ts;  % value of time for this step = last element in
                          % t vector + sample interval 
   t=[t  tnew];           % add to t vector for plotting - note that it would
                          % run faster it t were initialised to zeros first.
   error=uop-nldamp*xold(1)*xold(2)-2*xold(1)^3;   % first integrator i/p
   xnew=phi*xold+del*error;   % propagate states using discrete time model
   y=[y  xnew];               % add new values to output matrix for plotting.
                              % See comments on execution speed above.
   xold=xnew;             % update the state vector for the next time step
end
%
subplot(221)    % cause plotting in top left hand quarter of screen
plot(t,y),grid  % plot outputs = states vs. t
%
% Now repeat the full nonlinear simulation, but this time with a -0.1 unit
% deviation from the steady operating point:
%
yinit=[0.7937  0]'; % initial conditions = steady state operating point
y=yinit;      % begins a matrix y which we shall use to hold the outputs
              % at every time step for plotting
ustep=-0.1; % extra input superimposed upon steady operating
            % point of uop = 1
xold=y;     % initial state values to start the simulation
t=0;        % initial time value
tnew=0;     % tnew will be used to calculate the time at each step
%
% simulation FOR FIGURE 14.12(b):
disp('')
disp('Calculating Fig. 14.12(b)')
while tnew < tend
   tnew=t(length(t))+ts;
   t=[t  tnew];  % store time for plotting
   error=(uop+ustep)-nldamp*xold(1)*xold(2)-2*xold(1)^3;  % i/p to linear part
   xnew=phi*xold+del*error;
   y=[y  xnew];
   xold=xnew;
end
subplot(222)            % plot in top right quarter of screen
plot(t,y),grid,hold on  % hold on screen to plot linearised version on top
%
% generate linearized model response:
%
disp('')
disp('Calculating Fig. 14.12(c)')
jx=[0  1; -3.78  -1.19];  ju=[0  1]';
%
% give the model a series gain of ustep to allow use of unit step routine
% the series command connects two state space models in series. The state
% space model [], [], [], d represents a constant gain matrix d (a, b and
% c matrices being empty).
[al,bl,cl,dl]=series([],[],[],ustep,jx,ju,c,d);
[yl,xl,tl]=step(al,bl,cl,dl,1);
plot(tl,yl)  % overplots the existing nonlinear responses
%
% Now add the operating point offsets to the linear response to compare:
% PRODUCES FIGURE 14.12(c)
hold off      % release plot window
subplot(223)  % plot in lower left quarter of screen
plot(t,y),grid,hold on    % nonlinear result, as before
plot(tl,yl(:,1)+yinit(1),tl,yl(:,2)+yinit(2)) % linear results offset by yinit
%
% Now repeat for a deviation of, say, +0.5 units from the operating point:
%
yinit=[0.7937  0]';
y=yinit;
ustep= 0.5; % extra input superimposed upon operating point of uop = 1
%ustep= -1; % extra input superimposed upon operating point of uop = 1
xold=y;
t=0;
tnew=0;
%
% simulation FOR FIGURE 14.12(d):
disp('')
disp('Calculating Fig. 14.12(d)')
while tnew < tend
   tnew=t(length(t))+ts;
   t=[t  tnew];
   error=(uop+ustep)-nldamp*xold(1)*xold(2)-2*xold(1)^3; % i/p to linear part
   xnew=phi*xold+del*error;
   y=[y  xnew];
   xold=xnew;
end
hold off                % release plot window
subplot(224)            % plot in lower right quarter of screen
plot(t,y),grid,hold on  % hold to plot linearised version on top of it
%
% linear model response:
%
jx=[0  1; -3.77976  -1.19055];  ju=[0  1]';
%
% give the model a series gain of ustep to allow use of unit step routine
[al,bl,cl,dl]=series([],[],[],ustep,jx,ju,c,d);
[yl,xl,tl]=step(al,bl,cl,dl,1);
%
% Now add the operating point offsets to the linear response to compare:
% PRODUCES FIGURE 14.12(d)
disp('')
disp('plotting now - press any key after viewing plot')
plot(tl,yl(:,1)+yinit(1),tl,yl(:,2)+yinit(2)) % linear results offset by yinit
pause
%
% matlab 4.2 print:
%orient landscape
%print -dhpgl fig14_12
%
% matlab 3.5 print:
%meta fig14_12
%!gpp fig14_12 /dhpgl /ol
hold off
clg 