% Digital PID controllers for Example 5.12
% Generates Fig. 5.21
%

% Bill Barraclough 1994
% Ken Dutton 1996

clear y t ee u  % ensure no 'old' data around

% initialise:
ts=0.1;     % edit this to try different sampling interval
tmax=5;     % 5 second simulation

% set up time axis:
t=[0:ts:tmax];
% find number of time steps:
nmax=length(t)-1;

% s-domain plant transfer function:
nump=10;
denp=[1  8  17  10];

% Convert to discrete-time, so as to allow simulation
% with discrete controller.  For discrete controller and
% continuous plant, try SIMULINK.
[nzp,dzp]=c2dm(nump,denp,ts,'zoh');


% Explicit-integration p.i.d. simulation
% **************************************

% PID parameters:
kp=3;
td=0.333;
ti=1.5;
% Work out the controller constants (Eq. 5.26):
a = kp*(1 + td/ts + 0.5*ts/ti);
b = kp*(-td/ts + 0.5*ts/ti);
c = kp/ti;

% initialise simulation
pint=0;     % 'previous integral'
lastu=0;    % u at last time step
last1u=0;   % u at last time step but 1
lasty=0;    % etc.
last1y=0;
last2y=0;
laste=0;
y(1)=0;     % initial output

for n=1:nmax       % begin iteration
	e=1-lasty;  % error = unit step - o/p
	ee(n)=e;
	u(n) = a*e + b*laste + c*pint;   % PID

        % plant simulation
	y(n+1) = nzp(2)*u(n) + nzp(3)*lastu + nzp(4)*last1u;
	y(n+1) = y(n+1) - dzp(2)*lasty - dzp(3)*last1y - dzp(4)*last2y;

        % update 'previous integral'
	pint=pint + 0.5*ts*(e+laste);

        % update other parameters:
	laste=e;
	prv(n)=pint;
	last2u=last1u;
	last1u=lastu;
	lastu=u(n);
	last2y=last1y;
	last1y=lasty;
	lasty=y(n+1);
end

plot(t,y),grid

hold on  % freeze plot on screen whilst adding others

% 'Simple method' PID controller
% ******************************

% z-domain controller transfer function
numdc=[13.2  -23  10];
dendc=[1  -1   0];

% form the forward path by multiplying the plant and controller zTF:
numfp=conv(numdc,nzp);  % the conv(olution) command multiplies out
denfp=conv(dendc,dzp);  % polynomial coefficients

% add unity negative feedback loop:
[numcl,dencl]=cloop(numfp,denfp,-1);


% simulate and plot:
% Just the dstep command would do if an x-axis in sample numbers
% was adequate.  To get proper time axis, store values and plot
% separately against t:
[y,x]=dstep(numcl,dencl,nmax+1);
plot(t,y),grid


% Tustin method PID controller
% ******************************

% z-domain controller transfer function
numdc=[23.1  -39.8  17.1];
dendc=[1  0  -1];

% form the forward path by multiplying the plant and controller zTF:
numfp=conv(numdc,nzp);  % the conv(olution) command multiplies out
denfp=conv(dendc,dzp);  % polynomial coefficients

% add unity negative feedback loop:
[numcl,dencl]=cloop(numfp,denfp,-1);


% simulate and plot:
% Just the dstep command would do if an x-axis in sample numbers
% was adequate.  To get proper time axis, store values and plot
% separately against t:
[y,x]=dstep(numcl,dencl,nmax+1);
plot(t,y),grid


% Analogue version
% ****************

% add 'analogue' plot for comparison
ta=[0:0.1:tmax];
nfl=conv([1  3  2],nump);      % Laplace forward path numerator
dfl=conv([1  0],denp);         % Laplace forward path denominator
[ncll,dcll]=cloop(nfl,dfl,-1); % Laplace closed-loop TF
[ya,xa]=step(ncll,dcll,ta);
plot(ta,ya),grid

hold off         % release plot window




disp(' ')
disp(' Four traces are shown. See Fig. 5.21')
disp(' for which is which, or comment out the')
disp(' ''plot'' commands for those you do not want')
disp(' ')


