% Creates responses for Fig. 7.8
%
% System is plant of G(z) as given by Eq. [7.3]
% with deadbeat digital controller of D(z) given by Example 7.5.
% Unity negative feedback is applied.
%

% Ken Dutton 1996

% Simulate with the plant simulated much more
% 'continuously' than the controller.  SIMULINK is much easier
% for this, but here's the MATLAB version (it's an 
% inefficient way to use MATLAB):

% The system LTF is G(s) = Y(s)/U(s) = 8/(s^2 + 2s + 4).
% This converts to a difference equation by replacing
% dy/dt with (y(n) - y(n-1))/ti  (where ti is the integration
%      step length, and ti << T), and replacing
% d^2y/dt^2 with  (dy/dt(n) - dy/dt(n-1))/ti
%    = (y(n) - 2y(n-1) + y(n-2)) / (ti)^2
%
% The result is:
%
% y(n) = a.y(n-1) + b.y(n-2) + c.u(n)
%
% where  a = (2 + 2ti) / (1 + 2ti + 4(ti)^2)
%        b = -1 / (1 + 2ti + 4(ti)^2)
%        c = 8(ti)^2 / (1 + 2ti + 4(ti)^2)
%
% The discrete-time controller is executed every T seconds, where
% T = 0.4 second, according to the difference equation arising from
% Example 7.3 for D(z), namely:
%
% u(n) = 0.2376u(n-1) + 0.7624u(n-2)
%                     + 2.1094e(n) - 2.176e(n-1) + 0.9478e(n-2)
%

% initialise:

ti = 0.004; % 4ms integration step for continuous plant
T = 0.4;    % 400ms sampling period for digital controller
ns = T/ti;  % number of integration steps per controller sample
tend = 4;   % simulation duration (seconds)
a = (2 + 2*ti)/(1 + 2*ti + 4*ti*ti);  % parameters for 'continuous'
b = -1 / (1 + 2*ti + 4*ti*ti);        % part of simulation
c = 8*ti*ti / (1 + 2*ti + 4*ti*ti);
r = 1;        % unit step input always = 1
last_y = 0;   % previous output, y(n-1)
last1_y = 0;  % last but one output, y(n-2)
last_u = 0;   % etc.
last1_u = 0;
last_e = 0;
last1_e = 0;
tlast=0;      % time

count=ns;     % forces a controller evaluation on the first iteration

for n = 1:tend/ti    % start integration iterations
   t(n)=tlast+ti;    % build up time axis - purely for plot
   tlast=t(n);

   if count == ns  % execute a controller step
      count = 1;  % reset count
      e = r - last_y; % form error (one integration step unavoidable delay)
      u(n) = 0.2376*last_u + 0.7624*last1_u;
      u(n) = u(n) + 2.1094*e - 2.176*last_e + 0.9478*last1_e;
      last1_e = last_e; % update
      last_e = e;
      last1_u = last_u;
      last_u = u(n);
   else
      count = count + 1;  % no controller change this step
      u(n)=last_u;        % 'hold' on controller output
   end
   % simulate plant every integration step
   y(n) = a*last_y + b*last1_y + c*u(n);
   last1_y = last_y;  % update
   last_y = y(n);
end

plot(t,y),grid  % plot plant output against time axis
hold on         % freeze plot on screen to add some lines:
for k=0:T:tend;
   plot([k k],[0  1])  % verticals at sampling instants
end
plot([0  tend],[1  1]) % steady o/p level
hold off               % release plot window

disp(' ')
disp(' Output is displayed.')
disp(' Press a key to see control signal')
disp(' ')
pause

% MATLAB v4.2 hardcopy:
%orient landscape
%print fig7_6a -dhpgl

stairs(t,u),grid   % stairs gives true discrete-time plot

% MATLAB v4.2 hardcopy:
%orient landscape
%print fig7_6b -dhpgl

