% Creates responses for Fig. 7.7
%
% System is plant of G(z) as given by Eq. [7.3]
% with Kalman digital controller of D(z) given by Example 7.4.
% Unity negative feedback is applied.
%

% Copyright (c) Ken Dutton 1996

% Simulate with the plant simulated much more
% 'continuously' than the controller.  SIMULINK is much easier
% for this, but here's the MATLAB version (it's an 
% inefficient way to use MATLAB):

% The system LTF is G(s) = Y(s)/U(s) = 8/(s^2 + 2s + 4).
% This converts to a difference equation by replacing
% dy/dt with (y(n) - y(n-1))/ti  (where ti is the integration
%      step length, and ti << T), and replacing
% d^2y/dt^2 with  (dy/dt(n) - dy/dt(n-1))/ti
%    = (y(n) - 2y(n-1) + y(n-2)) / (ti)^2
%
% The result is:
%
% y(n) = a.y(n-1) + b.y(n-2) + c.u(n)
%
% where  a = (2 + 2ti) / (1 + 2ti + 4(ti)^2)
%        b = -1 / (1 + 2ti + 4(ti)^2)
%        c = 8(ti)^2 / (1 + 2ti + 4(ti)^2)
%
% The discrete-time controller is executed every T seconds, where
% T = 0.4 second, according to the difference equation arising from
% Example 7.4 for D(z), namely:
%
% u(n) = 0.567u(n-1) + 0.433u(n-2)
%                     + 1.197e(n) - 1.235e(n-1) + 0.538e(n-2)
%

% initialise:

disp(' ')
disp(' This program generates a lot of data.')
disp(' Please be patient!')
disp(' ')
disp(' In the event of an error message, clear')
disp(' the workspace and re-run')
disp(' ')
pause(1)

ti = 0.0005; % 0.5ms integration step for continuous plant
             % this fine a step is necessary to catch the controller
             % switching instants correctly.  Total number of data
             % points is 6000 - only every 50th is plotted. ti can
             % be lengthened, but only at the expense of accuracy.

T = 0.4;    % 400ms sampling period for digital controller
ns = T/ti;  % number of integration steps per controller sample
tend = 3;   % simulation duration (seconds)
np = 40;    % only every np th point to be plotted

a = (2 + 2*ti)/(1 + 2*ti + 4*ti*ti);  % parameters for 'continuous'
b = -1 / (1 + 2*ti + 4*ti*ti);        % part of simulation
c = 8*ti*ti / (1 + 2*ti + 4*ti*ti);
r = 1;        % unit step input always = 1
last_y = 0;   % previous output, y(n-1)
last1_y = 0;  % last but one output, y(n-2)
last_u = 0;   % etc.
last1_u = 0;
last_e = 0;
last1_e = 0;
tlast=0;      % time

count=ns;     % forces a controller evaluation on the first iteration
pcount=np;    % force plotted point on first iteration
nplt=1;
for n = 1:tend/ti    % start integration iterations

   if count == ns  % execute a controller step
      count = 1;   % reset count
      e = r - last_y; % form error (one integration step unavoidable delay)
      un = 0.567*last_u + 0.433*last1_u;
      un = un + 1.197*e - 1.235* last_e + 0.538*last1_e;
      last1_e = last_e; % update
      last_e = e;
      last1_u = last_u;
      last_u = un;
   else
      count = count + 1;  % no controller change this step
      un=last_u;          % 'hold' on controller output
   end
   % simulate plant every integration step
   yn = a*last_y + b*last1_y + c*un;
   last1_y = last_y;  % update
   last_y = yn;

   if pcount == np;  % point to be plotted this time
      t(nplt) = tlast+ti*np;  % build up time axis - purely for plot
      tlast = t(nplt);
      u(nplt) = un;
      y(nplt) = yn;
      nplt = nplt+1;
      pcount = 0;
   else
      pcount = pcount+1;  % no plotting this time
   end
end

plot(t,y),grid  % plot plant output against time axis

disp(' ')
disp(' Output is displayed.')
disp(' Press a key to see control signal')
disp(' ')
pause

% MATLAB v4.2 hardcopy:
%orient landscape
%print fig7_7a -dhpgl

stairs(t,u),grid   % stairs gives true discrete-time plot

% MATLAB v4.2 hardcopy:
%orient landscape
%print fig7_7b -dhpgl
