% Calculates the frequency response from a 127-bit impulse
% response rxy.  See text around Equation 8.7 in Chapter 8
%
% In the process, Figs. 8.6 and 8.8 are also created

% Ken Dutton 1996 and Bill Barraclough 1994

% firstly, generate the impulse response data on which the
% program can work.  At the end, the identified system should
% match the one we begin with here!

% Plant Laplace transfer function:
num=50;
den=[1  6  25];
ts=0.02;                      % sampling interval
[numd,dend]=c2dm(num,den,ts); % discrete-time model
lasty=0;     % initialise previous output value
last1y=0;    % and last but one output value
last1u=0;    % same for input
lastu=0;

% Now generate a 127 bit PRBS having levels +/- 2V
% Generates a PRBS vector 127 bits long called prbs
% First, set up a shifting matrix made up of a
% 6x6 identity matrix with an added top row of zeros
% and an added right hand column of zeros:

shif=[[0 0 0 0 0 0; eye(6)]  [0 0 0 0 0 0 0]'];

% (the effect of premultiplying a column 7-vector by this
% is to shift the first six elements down one place
% and put zero into element 1)

reg=[1 1 1 1 1 1 1]';  % initial shift register contents
for n=1:127;
   temp=reg(1)+reg(7);
   if temp==2,temp=0;end;
   reg=shif*reg;    % shift elements 1 to 6 down 1
   reg(1)=temp;     % new element 1
   bits(n)=reg(7);  % end bit to PRBS sequence
   bits(n)=4*(bits(n)-.5);
end

ip=[bits bits bits bits bits]; % the input (5 PRBS cycles)

% now generate the plant output response to the PRBS:

for n=1:635;
   op(n)=numd(2)*lastu + numd(3)*last1u - dend(2)*lasty...
                - dend(3)*last1y;
   last1y=lasty;
   lasty=op(n);
   last1u=lastu;
   lastu=ip(n);
end

stairs(op),grid

disp(' ')
disp(' The plant output to the PRBS is displayed')
disp(' Press any key to continue after viewing it')
disp(' (The next part takes several seconds)')
pause


% Next, calculate the cross-correlation of two series
% ip and op, storing it in rxy.

n1=127;
n2=n1+n1;
for ii = 1 : n1+5;
   temp1=0;
   temp=0;
   for jj = n1+1 : n2; % perform shifting and adding
      temp=temp+ip(jj)*op(ii+jj-1);
      temp1=temp1+ip(jj)*ip(ii+jj-1);
   end
   % now do the averaging:
   rrxy(ii)=temp/n1;  % cross correlation of ip and op
   acor(ii)=temp1/n1; % autocorrelation of PRBS
end

rxy=rrxy-rrxy(n1);  % correction for zero error

t=[0:131]*ts;

plot(t,acor);grid;xlabel('Delay, second');...
ylabel('Autocorrelation of PRBS, volt ^2');

disp(' ')
disp(' The PRBS autocorreation is displayed')
disp(' Press any key to continue after viewing it')
pause

plot(t,rxy,t,rrxy);grid;xlabel('Delay, second');...
ylabel('Cross-correlation, volt ^2');...
title('Lower - uncorrected; Upper - corrected for zero error');

disp(' ')
disp(' The cross-correlation is displayed - Fig. 8.6')
disp(' Press any key to continue after viewing it')
pause


% Finally, generate the frequency response from the 
% impulse response.

w=logspace(-2,2,40);  % radian frequency values
for ii=1:40;          % step through frequency values
   fq=0;
   for k=1:127;       % step through impulse response values
      % do Equation 8.7:
      fq=fq+rxy(k)*exp(-w(ii)*ts*(k-1)*j);   % N.B. j = sqrt(-1)
   end
   ag(ii)=180/pi*angle(fq);  % phase values converted to degrees
   vl=abs(fq)/acor(1);       % magnitudes
   db(ii)=20*log10(vl);      % converted to dB
end

semilogx(w,db);grid;xlabel('Frequency, rad/s');...
ylabel('System gain in dB');

disp(' ')
disp(' The dB magnitude plot is displayed - Fig. 8.8')
disp(' Press any key to continue after viewing it')
pause


semilogx(w,ag);grid;xlabel('Frequency, rad/s');...
ylabel('Phase angle in degree');

disp(' ')
disp(' The phase plot is displayed')