% Fig11_4.m  Demonstrates on-line identification
% for a second-order plant

% Ken Dutton & Bill Barraclough, 1996

disp(' ')
disp(' If the program pauses, try looking for a plot in a')
disp(' hidden window')
disp(' ')
disp(' Press any key after viewing a plot')
disp(' ')
disp(' The calculations may take a while')
disp(' ')
disp(' Press any key after viewing these messages')
pause 

disp(' ')
disp('The mass m will be 1.0 kg initially.')
disp('It will increase to 5 kg after 20 s')
disp('and reduce to 0.2 kg after 40 s.')
disp(' ')

% Enter the plant LTF:
num=40;
den=[1 10 20];

% and convert to ARMA model with 50ms samples:
ts=0.05;  
[bp,ap]=c2dm(num,den,ts); % this is a z-transform model - now convert....

% At this point, we have:
%
%         Y(z)   0 + b1*z^(-1) + b2*z^(-2)
%  G(z) = ---- = -------------------------
%         U(z)   1 - a1*z^(-1) - a2*z^(-2)
%
% but the '0' is bp(1) and the '1' is ap(1).  This needs to be
% rearranged into ARMA form:
%
%   y = ap*(last two y values) + bp*(last two u values)
%
% so the '0' and '1' need removing from the coefficient vectors,
% and the signs of ap need changing as it's changed sides:

ap=-ap(2:length(ap));
bp=bp(2:length(bp));  

q=length(bp);  % order of the numerator = no. of terms in bp
p=length(ap);  % order of the denominator = no. of terms in ap

% For this run, the digital controller is NOT self-tuning (because we are
% only interested in identifying the plant at present), but it is still
% in the loop.  In the z-domain it is given by
% U(z)/E(z) = D(z) = db(z)/da(z).  This is cross-multiplied and
% rearranged into the ARMA form, so that the next value of u is given by
% u = da*(previous values of u) + db*(values of e).
% Thus, da has been modified compared with the z-transform, by
% having had the first term (unity) removed and taken to the left of the
% equation to give the ARMA model (and the signs of the other terms changed).

% In the analysis immediately before Example 11.4, D(z) is given as
%
%        U(z)   [1 - e^(-beta*ts)] (1 - a1*z^(-1)  - a2*z^(-2))
% D(z) = ---- = ----------------------------------------------
%        E(z)              (1 - z^(-1))(b1 + b2)
%
% so, following the discussion above, with   1 - e^(-beta*ts) = 0.1  (say):
%


da=[1 0];      % note the trailing zero (coeff. of z^(-2))
               % to equal the length of db
db=0.1*[1 -ap]/sum(bp);


% Initialise the covariance matrix (it is square, and of size p+q):
pcvm=eye(p+q)*1000;

% Initialise the vector theta (it is a column vector of length p+q):
% For this example, theta will contain:
% [a1;  a2;  b1;  b2]
theta=ones(p+q,1);

% The vector  x  used below will similarly contain
% [y(n-1);  y(n-2);  u(n-1);  u(n-2)]
% In order to update x at each time step, it must become:
% [0;  y(n-1);  0;  u(n-1)]  ready for the next values of y and u to
% be added.  This can be done by premultiplying  x  by:

shift=[0 0 0 0
       1 0 0 0
       0 0 0 0
       0 0 1 0];

% and then inserting the most recent measurement of y into element 1
% of x, and of u into element 3.

% find the numbers of time steps corresponding to 20 and 40 seconds,
% when the mass is to be changed:
t20=round(20/ts);
t40=round(40/ts);

% limit to 1200 steps:
if t20 > 400
   t20=400;
   t40=800;
end
isteps=3*t20;

% time values:
t=[1 : isteps]*ts;

% Initialise signal vectors (last two values in most cases)
y=0;
ev=[0 0]';     % vector of past errors
x=[0 0 0 0]';  % vector of past outputs stacked above vector of past inputs
    
nn=menu('Choose an input signal:','low-frequency square wave',...
                                  'single step','sinewave');
w=2*pi*0.1;  % radian frequency for 0.2 Hz
if nn==1, r=square(w*t);end        % data points for squarewave input
if nn==2, r=ones(1,isteps);end     % data points for single step input
if nn==3, r=sin(w*t);end           % data points for sinewave input


disp(' ')
disp(' Calculating responses - please wait')
disp(' ')
pause(1) % forces message to display

%	*** Now the run starts! ***
for i1=1:isteps
	if i1==t20   % switch mass to 5 kg at 20 seconds
		m=5;
		den=[m 10 20];
		[bp,ap]=c2dm(num,den,ts);
		ap=-ap(2:length(ap));  % extract 'a' coefficients
                bp=bp(2:length(bp));   % remove leading zero
	end
	if i1==t40   % switch mass to 0.2 kg at 40 seconds
		m=0.2;
		den=[m 10 20];
		[bp,ap]=c2dm(num,den,ts);
		ap=-ap(2:length(ap));  % extract 'a' coefficients
                bp=bp(2:length(bp));   % remove leading zero
	end


	e = r(i1)-x(1);  % form error in f/back loop (x(1) is previous output)
	u = da*x(3:4) + db*[e; ev];  % plant input from digital controller
	y=ap*x(1:2) + bp*x(3:4); % next plant output from 'true' ARMA model
        ev(2)=ev(1);     % update vector of past errors
	ev(1)=e;
	uu(i1)=u;        % store input value for plots
	yy(i1)=y;        % store output value for plots

% 	*** Now for the on-line identification ***
	a1(i1)=theta(1); % store last identified coefficients for plotting
	a2(i1)=theta(2);
	b1(i1)=theta(3);
	b2(i1)=theta(4);
	temp=x'*pcvm;
	temp1=1 + temp*x;
        temp2=x'*theta - y;
	if temp1~=0
           temp3=pcvm*x*temp2/temp1;
           theta=theta-temp3;          % parameter estimate update
           temp4=x*x';
           temp5=pcvm*temp4;
           temp6=temp5*pcvm/temp1;
	   pcvm=pcvm-temp6;            % covariance update
        end

	x=shift*x;       % update vector of past outputs over inputs
	x(1)=y;
	x(3)=u;

end
plot(t,[a1' a2' b1' b2']),grid

disp(' The estimated parameters were:')
disp(' ')
disp('For mass of 1kg:')
disp('      a1         a2        b1       b2')
te=t20-1;
[a1(te) a2(te) b1(te) b2(te)]
disp(' ')
disp('For mass of 5kg:')
disp('      a1         a2        b1       b2')
te=t40-1;
[a1(te) a2(te) b1(te) b2(te)]
disp(' ')
disp('For mass of 0.2kg:')
disp('      a1         a2        b1       b2')
te=length(a1);
[a1(te) a2(te) b1(te) b2(te)]
