% Digital PID controller simulation for Example 5.13.
% Generates elements of Fig. 5.23 and 5.24.
%
% This file is provided to demonstrate how a high-level
% language can be used to simulate mixed-sampling-rate
% systems.  It is, however, a very inefficient way to use
% MATLAB.  The worst-case element of Example 5.13 is used.
%
% Edit the controller transfer function and sampling rate
% to produce the other elements.
%
% Use the SIMULINK 2.2 models in fig5_23s.mdl and
% fig5_24s.mdl to do the example 'properly'.

% Ken Dutton 1999

disp(' ')
disp(' Use the SIMULINK 2.2 models in fig5_23s.mdl')
disp(' and fig5_24s.mdl to do the example ''properly''.')
disp(' ')
disp(' Further details are inside the m-file.  Please use')
disp(' ''help fig5_23'' and look at the file with an editor too.')
disp(' ')
disp(' Press a key after noting this message.')
disp(' ')
pause

clear % ensure no 'old' data around

% initialise controller parameters (change next two
% assignments for other controllers in Example 5.13):
tsc=0.5; % controller sampling interval (second).  This
         % should be a multiple of 0.01s for best results.
         % tsc=0.01 or 0.1 sec. have alternative discrete-
         % time p.i.d. numerators below
dtpidnm=[6  -7  2];  % z-domain pid numerator 
                     % for tsc=0.5 sec.
%dtpidnm=[13.2  -23  10];  % z-domain pid numerator
                           % for tsc=0.1 sec.
%dtpidnm=[103.02  -203  100];  % z-domain pid numerator
                               % for tsc=0.01 sec.
dtpiddn=[1 -1 0]; % denominator for all three pid models
                           
% For pid: U(z)/E(z)=dtpidnm(z)/dtpiddn(z), evaluation is:
% u = dtpidnm(1)e + dtpidnm(2)z^(-1)e + dtpidnm(3)z^(-2)e
%     + z^(-1)u

ts=0.01; % sampling interval for 'continuous-time' plant
         % simulation (stays the same for any pid model).
tmax=5;  % 5 second simulation

% s-domain plant transfer function:
nump=10;
denp=[1  8  17  10];
plant=tf(nump,denp);

% PID parameters (listed for ref. only):
%kp=3;
%td=0.333;
%ti=1.5;

% set up time axis:
t=[0:ts:tmax];
% find number of time steps:
nmax=length(t)-1;
t=t(1:nmax); % (correct length for plotting)
y=zeros(1,nmax); % plant output vector
u=zeros(1,nmax); % plant input vector
% Note that the controller output is, in theory, the
% same signal as the plant input, but actually needs
% to exist (as does the error signal) at the controller
% sampling rate, not the plant sampling rate.  
% The variables ezm1, ezm2 and uczm1 take care of this below.

% find number of basic simulation time steps per
% controller sampling interval:
stepcon=round(tsc/ts);

disp(' ')
disp([' Time steps per controller evaluation = ' num2str(stepcon)])

% Convert plant to discrete-time, so as to allow simulation:
dtplant=c2d(plant,ts,'zoh');
[numd,dend]=tfdata(dtplant,'v');  % extract z-domain coefficients
% Rearrange Y(z)/U(z)=numd/dend as:
% y=au + bz^(-1)u + cz^(-2)u + dz^(-3)u
%   - fz^(-1)y - gz^(-2)y - hz^(-3)y
% (everything needs dividing by dend(1) to get 1.y on LHS)
a=numd(1)/dend(1);  b=numd(2)/dend(1);
c=numd(3)/dend(1);  d=numd(4)/dend(1);
f=dend(2)/dend(1);  g=dend(3)/dend(1);  h=dend(4)/dend(1);

% initialise simulation variables (note that 'step' means the 
% controller sample time (tsc) for ezm1, ezm2 and uczm1; but
% the basic simulation time step (ts) for the other signals):
ezm1=0; % loop error at previous step (tsc)
ezm2=0; % loop error two steps ago (tsc)
uczm1=0; % controller output at previous step (tsc)
uzm1=0; % controller output at previous step (ts)
uzm2=0; % controller output two steps ago (ts)
uzm3=0; % controller output three steps ago (ts)
yzm1=0; % plant output at previous step (ts)
yzm2=0; % plant output two steps ago (ts)
yzm3=0; % plant output three steps ago (ts)
r=1;    % unit step reference input
pidcnt=stepcon; % forces pid output evaluation first loop

% main simulation loop counter
for k=1:nmax
   
   % pid controller:
   if pidcnt==stepcon
      % pid evaluation due at this step
      
      % loop error:
      e=r-yzm1;
      
      % pid controller
      u(k)=dtpidnm(1)*e + dtpidnm(2)*ezm1 + dtpidnm(3)*ezm2 + uczm1;
      
      pidcnt=0; %reset pid evaluation counter
      
      % update variables sampled at controller rate
	   ezm2=ezm1;
   	ezm1=e;
	   uczm1=u(k);
      
   else
      % pid output still held waiting for next controller sample
      u(k)=uzm1;
   end
   
   % plant:
   y(k)=a*u(k) + b*uzm1 + c*uzm2 + d*uzm3 - f*yzm1 - g*yzm2 - h*yzm3;
   
   %update variables sam;pled at plant rate (ts):
   uzm3=uzm2;
   uzm2=uzm1;
   uzm1=u(k);
   yzm3=yzm2;
   yzm2=yzm1;
   yzm1=y(k);
   pidcnt=pidcnt+1;
end

plot(t,y);grid

disp(' ')
disp(' Output (part of Fig. 5.23) is on screen')
disp(' Press a key to see controller output (Fig. 5.24)')
disp(' ')
pause

plot(t,u);grid

disp(' ')
disp(' Controller output (part of Fig. 5.23) is on screen')
disp(' ')
disp(' To see the other controllers in Figs 5.23 and 5.24')
disp(' please edit two lines in fig5_23.m where indicated.')
disp(' ')
