% Creates responses for Fig. 7.9
%
% System is plant of G(z) as given by Eq. [7.3]
% with Dahlin digital controller of D(z) with ringing
% pole removed, given at end of section 7.6.
% Unity negative feedback is applied.
%

% Ken Dutton 1996
% clear and close added for correct operation, 7.1.99


% Simulate with the plant simulated much more
% 'continuously' than the controller.  SIMULINK is much easier
% for this, but here's the MATLAB version (it's an 
% inefficient way to use MATLAB):

% The system LTF is G(s) = Y(s)/U(s) = 8/(s^2 + 2s + 4).
% This converts to a difference equation by replacing
% dy/dt with (y(n) - y(n-1))/ti  (where ti is the integration
%      step length, and ti << T), and replacing
% d^2y/dt^2 with  (dy/dt(n) - dy/dt(n-1))/ti
%    = (y(n) - 2y(n-1) + y(n-2)) / (ti)^2
%
% The result is:
%
% y(n) = a.y(n-1) + b.y(n-2) + c.u(n)
%
% where  a = (2 + 2ti) / (1 + 2ti + 4(ti)^2)
%        b = -1 / (1 + 2ti + 4(ti)^2)
%        c = 8(ti)^2 / (1 + 2ti + 4(ti)^2)
%
% The discrete-time controller is executed every T seconds, where
% T = 0.4 second, according to the difference equation arising from
% Example 7.3 for D(z), namely:
%
% u(n) = u(n-1) + 0.591e(n) - 0.609e(n-1) + 0.266e(n-2)
%

% initialise:
clear
close

ti = 0.004; % 4ms integration step for continuous plant
T = 0.4;    % 400ms sampling period for digital controller
ns = T/ti;  % number of integration steps per controller sample
tend = 6;   % simulation duration (seconds)

% the combination of ti and tend gives too much data.  Only
% plot every 10th point
np = 10;

a = (2 + 2*ti)/(1 + 2*ti + 4*ti*ti);  % parameters for 'continuous'
b = -1 / (1 + 2*ti + 4*ti*ti);        % part of simulation
c = 8*ti*ti / (1 + 2*ti + 4*ti*ti);
r = 1;        % unit step input always = 1
last_y = 0;   % previous output, y(n-1)
last1_y = 0;  % last but one output, y(n-2)
last_u = 0;   % etc.
last1_u = 0;
last_e = 0;
last1_e = 0;
tlast=0;      % time

count=ns;     % forces a controller evaluation on the first iteration
pcount=np;    % force plotted point on first iteration
nplt=1;

for n = 1:tend/ti    % start integration iterations
    if count == ns  % execute a controller step
      count = 1;  % reset count
      e = r - last_y; % form error (one integration step unavoidable delay)
      un = last_u + 0.591*e - 0.609*last_e + 0.266*last1_e;
      last1_e = last_e; % update
      last_e = e;
      last1_u = last_u;
      last_u = un;
   else
      count = count + 1;  % no controller change this step
      un=last_u;        % 'hold' on controller output
   end
   % simulate plant every integration step
   yn = a*last_y + b*last1_y + c*un;
   last1_y = last_y;  % update
   last_y = yn;

  if pcount == np;  % point to be plotted this time
      t(nplt) = tlast+ti*np;  % build up time axis - purely for plot
      if nplt==1, t(nplt)=0; end  % correct first value
      tlast = t(nplt);
      u(nplt) = un;
      y(nplt) = yn;
      nplt = nplt+1;
      pcount = 1;
   else
      pcount = pcount+1;  % no plotting this time
   end
end

plot(t,y),grid  % plot plant output against time axis


disp(' ')
disp(' Output is displayed.')
disp(' Press a key to see control signal')
disp(' ')
pause

% MATLAB 4.2 hardcopy:
%orient landscape
%print fig7_9a -dhpgl

axis([0  6  0  0.6]);  % for MATLAB v3.5
stairs(t,u),grid   % stairs gives true discrete-time plot
axis([0  6  0  0.6]); % for MATLAB v4.2

% MATLAB 4.2 hardcopy:
%orient landscape
%print fig7_9b -dhpgl

