function [ka,kb,kc,kd] = mfacc(w,f,index,snum,sden)
%MFACC   Synthesise an approximate commutative controller.
%
%       [ka,kb,kc,kd] = mfacc(w,f,index,snum,sden)
%
%       f is the MVFR matrix of a square system and w is the 
%       associated frequency list. The approximate commutative 
%       controller is designed at frequency w(index). 
%       (index should be a single integer.)
%       The i'th row of snum and the i'th row of sden contain the 
%       numerator and denominator, respectively, of the i'th diagonal
%       compensator of the controller, namely the compensator applied
%       to the i'th characteristic locus.
%
%       ka,kb,kc,kd are the state space matrices of the resulting 
%       controller.

%       Dr J.M. Maciejowski,  20 November 1987. Revised 7 Dec 1987.
% Copyright (c) 1987-1997 Cambridge Control Ltd

% Renamed from facc.m to allow edits, original saved as
% facc.org, Ken Dutton, 6.1.99
% align changed to mvalign, Ken Dutton, 6.1.99
% max(size(ka)) changed to  length(ka), Ken Dutton, 6.1.99
% Modified version supplied by agreement with Cambridge Control Ltd.
% and intended for use only with textbook examples from "The Art
% of Control Engineering" by Dutton, Thompson and Barraclough.
% See Section 5.1 of the "readme5.txt" file which accompanies the
% set of m-files, to see other required changes to the mfd toolbox.

% History:
% Matlab 5 compatibility (JMM):
% Initialized ka,kb,kc,kd, 20.8.97.
% max(size(ai)) changed to size(ai,1), 1 place, 20.8.97.
% `if abcpgchk(...)' changed to `pg=abcpgchk(...); if pg,...', 22.8.97.

% Initialize variables:
ka = []; kb = []; kc = []; kd = [];

[m,n]=fsize(w,f);
if m~=n
  error('Not a square system. (FACC)')
end

% Check consistency of dimensions:
[nr,nc]=size(snum);
if nr ~= n
  error('Incorrect number of numerators. (FACC)')
end
[dr,dc]=size(sden);
if dr~=n
  error('Incorrect number of denominators. (FACC)')
end

% First get a state space model of the diagonal part:
for i=1:n
  % Remove leading zeros:
  sni = snum(i,min(find(snum(i,:)~=0)):length(snum(i,:)));
  sdi = sden(i,min(find(sden(i,:)~=0)):length(sden(i,:)));
  % Get state space model of i'th diagonal:
  [ai,bi,ci,di]=tf2ss(sni,sdi);
  pg  = abcpgchk(ai,bi,ci); % Deal with pure gains
  if pg,
    ai=[]; bi=[]; ci=[];
  end
  ni=size(ai,1); % Modified 20.8.97 for v5 compatibility
  ka = [ka,  zeros(length(ka),ni);
	zeros(ni,length(ka)), ai  ];
  [br,bc]=size(kb);
  kb = [kb, zeros(br,1);
	zeros(ni,bc), bi ]; 
  [cr,cc]=size(kc);
  kc = [kc, zeros(cr,ni);
	zeros(1,cc), ci  ];  
  kd = [kd, zeros(cr,1);
	zeros(1,bc), di ];
end
% Assertion: kb,kd have n columns, kc,kd have n rows.
%
% Now find real approximations to the eigenvector frames:
[v,d]=eig(fgetf(w,f,index));
wr=mvalign(v);
vr=mvalign(inv(v));  % vr=inv(wr) may work better sometimes
%
% Now form series connection vr*s*wr:
kb=kb*wr;
kc=vr*kc;
kd=vr*kd*wr;
%
% Uncomment next line to ensure minimal realization:
% [ka,kb,kc,kd]=minreal(ka,kb,kc,kd);
